MainWindow = class(Turbine.UI.Lotro.Window);

settings = {position = {left = 100, top = 100}, 
size = {width = 400, height = 400},
lyrics = {[[
Gil-galad was an Elven-king,
Of him the harpers sadly sing:
The last whose realm was fair and free
Between the mountains and the sea.

His sword was long, his lance was keen.
His shining helm afar was seen.
The countless stars of heaven's field
Were mirrored in his silver shield.

But long ago he rode away,
And where he dwelleth none can say.
For into darkness fell his star;
In Mordor, where the shadows are.]]}, page = 1};

function Turbine.Plugin:Unload(sender, args)
	settings.position.left = mainWindow:GetLeft();
	settings.position.top = mainWindow:GetTop();
	settings.size.width = mainWindow:GetWidth();
	settings.size.height = mainWindow:GetHeight();
	settings.lyrics = mainWindow.lyrics;
	settings.page = mainWindow.page;
	Turbine.PluginData.Save(Turbine.DataScope.Account, "Lyrical", settings);
end

function MainWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor(self);

	settings = Turbine.PluginData.Load(Turbine.DataScope.Account, "Lyrical") or settings;

	self.line = 1;
	self.editmode = false;
	self.lyrics = settings.lyrics;
	self.page = settings.page;
	self:SetPosition(settings.position.left, settings.position.top);
	self:SetSize(settings.size.width, settings.size.height);
	self:SetText("Lyrical");

	-- lyrics box
	self.lyricsBox = LyricsBox(self);
	self.lyricsBox:SetParent(self);
	self.lyricsBox:SetPosition(20, 35 + 30);
	self.lyricsBox.SelectedIndexChanged = function(sender, args)
		self:Select(self.lyricsBox:GetSelectedIndex());
	end
	
	-- vertical scrollbar
	self.scrollBar1 = Turbine.UI.Lotro.ScrollBar();
	self.scrollBar1:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.scrollBar1:SetParent(self);
	self.lyricsBox:SetVerticalScrollBar(self.scrollBar1);

	-- edit box
	self.editBox = Turbine.UI.Lotro.TextBox();
	self.editBox:SetParent(self);
	self.editBox:SetPosition(20, 35 + 30);
	self.editBox:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	self.editBox:SetText("");
	self.editBox:SetVisible(false);

	-- vertical scrollbar
	self.scrollBar2 = Turbine.UI.Lotro.ScrollBar();
	self.scrollBar2:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.scrollBar2:SetParent(self);
	self.editBox:SetVerticalScrollBar(self.scrollBar2);
	self.scrollBar2:SetVisible(false);

	-- edit button
	self.editButton = Turbine.UI.Lotro.Button();
	self.editButton:SetParent(self);
	self.editButton:SetPosition(20, 35);
	self.editButton:SetSize(60, 20);
	self.editButton:SetText("Edit");
	self.editButton.MouseClick = function(sender, args)
		if not self.editmode then
			self.editBox:SetText(self.lyrics[self.page]);
			self.editButton:SetText("Save");
			self.newButton:SetEnabled(false);
			self.pageLeft:SetEnabled(false);
			self.pageRight:SetEnabled(false);
			self.deleteButton:SetEnabled(false);
			self.lyricsBox:ClearItems();
			self.lyricsBox:SetVisible(false);
			self.scrollBar1:SetVisible(false);
			self.editBox:SetVisible(true);
			self.scrollBar2:SetVisible(true);
			self.editmode = true;
		else
			self.lyrics[self.page] = self.editBox:GetText();
			self:LoadPage(self.page);
			self.editButton:SetText("Edit");
			self.newButton:SetEnabled(true);
			self.lyricsBox:SetVisible(true);
			self.scrollBar1:SetVisible(true);
			self.editBox:SetVisible(false);
			self.scrollBar2:SetVisible(false);
			self.editmode = false;
		end
	end

	-- new button
	self.newButton = Turbine.UI.Lotro.Button();
	self.newButton:SetParent(self);
	self.newButton:SetPosition(20 + 60, 35);
	self.newButton:SetSize(60, 20);
	self.newButton:SetText("New");
	self.newButton.MouseClick = function(sender, args)
		if self.page == #self.lyrics then
			self.page = self.page + 1;
		else
			for i = #self.lyrics, self.page, -1 do
				self.lyrics[i+1] = self.lyrics[i];
			end
		end
		self.lyrics[self.page] = "";
		self:LoadPage(self.page);
	end

	-- delete button
	self.deleteButton = Turbine.UI.Lotro.Button();
	self.deleteButton:SetParent(self);
	self.deleteButton:SetPosition(20 + 120, 35);
	self.deleteButton:SetSize(60, 20);
	self.deleteButton:SetText("Delete");
	self.deleteButton.MouseClick = function(sender, args)
		if self.page == #self.lyrics then
			self.lyrics[self.page] = nil;
			self.page = self.page - 1;
		else
			for i = self.page, #self.lyrics - 1 do
				self.lyrics[i] = self.lyrics[i+1];
			end
			self.lyrics[#self.lyrics] = nil;
		end
		self:LoadPage(self.page);
	end

	-- page controls
	self.pageLeft = Turbine.UI.Lotro.Button();
	self.pageLeft:SetParent(self);
	self.pageLeft:SetSize(40, 20);
	self.pageLeft:SetText("<");
	self.pageLeft.MouseClick = function(sender, args)
		self.page = self.page - 1;
		self:LoadPage(self.page);
	end

	self.pageRight = Turbine.UI.Lotro.Button();
	self.pageRight:SetParent(self);
	self.pageRight:SetSize(40, 20);
	self.pageRight:SetText(">");
	self.pageRight.MouseClick = function(sender, args)
		self.page = self.page + 1;
		self:LoadPage(self.page);
	end

	self.pageLabel = Turbine.UI.Label();
	self.pageLabel:SetParent(self);
	self.pageLabel:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	self.pageLabel:SetForeColor(Turbine.UI.Color(0.8, 0.8, 0.8));
	self.pageLabel:SetSize(40, 20);
	self.pageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);

	-- resize control
	self.resizeCtrl = Turbine.UI.Control();
	self.resizeCtrl:SetParent(self);
	self.resizeCtrl:SetSize(20, 20);
	self.resizeCtrl:SetPosition(self:GetWidth() - 20, self:GetHeight() - 20);
	self.resizeCtrl.MouseDown = function(sender, args)
		sender.dragStartX = args.X;
		sender.dragStartY = args.Y;
		sender.dragging = true;
	end
	self.resizeCtrl.MouseUp = function(sender, args)
		sender.dragging = false;
	end;
	self.resizeCtrl.MouseMove = function(sender, args)
		local width, height = self:GetSize();
		if (sender.dragging) then
			self:SetSize(width + (args.X - sender.dragStartX),
				height + (args.Y - sender.dragStartY));
			if (self:GetWidth() < 350) then
				self:SetWidth(350);
			end
			if (self:GetHeight() < 138) then
				self:SetHeight(138);
			end
			self:Layout();
		end
		sender:SetPosition(self:GetWidth() - sender:GetWidth(),
			self:GetHeight() - sender:GetHeight());
	end

	self:Layout();
	self:LoadPage(self.page);
end

function MainWindow:Layout()
	self.lyricsBox:SetSize(self:GetWidth() - (20 * 2) - 10, self:GetHeight() - 65 - 35 - 20);

	self.scrollBar1:SetSize(10, self.lyricsBox:GetHeight());
	self.scrollBar1:SetPosition(self.lyricsBox:GetLeft() + self.lyricsBox:GetWidth(), self.lyricsBox:GetTop());

	self.editBox:SetSize(self:GetWidth() - (20 * 2) - 10, self:GetHeight() - 65 - 35 - 20);

	self.scrollBar2:SetSize(10, self.lyricsBox:GetHeight());
	self.scrollBar2:SetPosition(self.lyricsBox:GetLeft() + self.lyricsBox:GetWidth(), self.lyricsBox:GetTop());

	self.pageLeft:SetPosition(self:GetWidth() - 20 - 40 * 3 - 10, 35);
	self.pageRight:SetPosition(self:GetWidth() - 20 - 40 - 10, 35);
	self.pageLabel:SetPosition(self:GetWidth() - 20 - 40 * 2 - 10, 35);

	self.lyricsBox:Layout();
end

function MainWindow:Destructor()
	self.lyricsBox:ClearItems();
end

function MainWindow:Select(i)
	if self.lyricsBox:GetItem(i).enabled then
		self.lyricsBox:GetItem(self.line):SetSelected(false);
		self.line = i;
		self.lyricsBox:GetItem(self.line):SetSelected(true);
	end
end

function MainWindow:Advance()
	local i = self.line;
	while true do
		i = i + 1;
		if i > self.lyricsBox:GetItemCount() then
			i = 0;
		elseif self.lyricsBox:GetItem(i).enabled then
			break;
		end
	end
	self:Select(i);
end

function MainWindow:Init()
	local i = 1;
	while true do
		if i > self.lyricsBox:GetItemCount() then
			return;
		elseif self.lyricsBox:GetItem(i).enabled then
			break;
		end
		i = i + 1;
	end
	self.line = i;
	self:Select(i);
end

function MainWindow:LoadPage(page)
	self.page = page;
	self.lyricsBox:Load(self.lyrics[self.page]);
	self.pageLabel:SetText(self.page .. "/" .. #self.lyrics);
	if self.page == 1 then
		self.pageLeft:SetEnabled(false);
	else
		self.pageLeft:SetEnabled(true);
	end
	if self.page == #self.lyrics then
		self.pageRight:SetEnabled(false);
	else
		self.pageRight:SetEnabled(true);
	end
	if #self.lyrics == 1 then
		self.deleteButton:SetEnabled(false);
	else
		self.deleteButton:SetEnabled(true);
	end
	self:Init();
end
